package com.cedarsoft.hudson.bears;

import org.apache.commons.io.IOUtils;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.MalformedURLException;

/**
 * The bear updater
 */
public class BearUpdater {
  public static void main( String[] args ) throws Exception {
    if ( args.length == 0 ) {
      throw new IllegalArgumentException( "Need two parameters: <URL to Hudson> <Host/IP of plug connector>" );
    }

    String hudsonUrl = args[0].trim();
    String hostPlugConnector = args[1].trim();

    new BearUpdater( hudsonUrl, hostPlugConnector ).run();
  }

  @NonNls
  @NotNull
  private final StatusUrls statusUrls;
  @NotNull
  private final PlugConnector plugConnector;

  private volatile boolean shuttingDown;

  public BearUpdater( @NotNull @NonNls String hudsonUrl, @NotNull @NonNls String hostPlugConnector ) throws MalformedURLException {
    statusUrls = new StatusUrls( hudsonUrl );
    plugConnector = new PlugConnector( hostPlugConnector );

    System.out.println( "Starting Bear Updater..." );
    System.out.println( "\tConnecting to Hudson @ <" + hudsonUrl + ">" );
    System.out.println( "\tConnecting to Plug Connector @ <" + hostPlugConnector + ">" );
  }

  private void run() throws IOException {
    Runtime.getRuntime().addShutdownHook( new Thread( new Runnable() {
      public void run() {
        shuttingDown = true;
        System.out.println( "Disabling all" );
        try {
          plugConnector.disableAll();
        } catch ( IOException e ) {
          e.printStackTrace();
        }
      }
    } ) );

    while ( true ) {
      if ( Thread.currentThread().isInterrupted() ) {
        return;
      }

      ProjectsState projectsState = parseProjectsState();
      if ( shuttingDown ) {
        return;
      }
      switchBears( projectsState );

      try {
        Thread.sleep( 1000 * 60 );
      } catch ( InterruptedException ignore ) {
      }
    }
  }

  private void switchBears( @NotNull ProjectsState criticalState ) throws IOException {
    PlugState[] plugStates = {PlugState.OFF, PlugState.OFF, PlugState.OFF, PlugState.OFF};

    if ( criticalState.hasBuildingProject() ) {
      plugStates[0] = PlugState.ON;
    }

    if ( criticalState.hasFailedProject() ) {
      plugStates[3] = PlugState.ON;
    } else {
      if ( criticalState.hasInstableProject() ) {
        plugStates[2] = PlugState.ON;
      } else {
        if ( criticalState.hasStableProject() ) {
          plugStates[1] = PlugState.ON;
        }
      }
    }
    plugConnector.switchPlugs( plugStates );
  }

  @NotNull
  private ProjectsState parseProjectsState() throws IOException {
    String statusXml = IOUtils.toString( statusUrls.getCritical().openStream() );

    ProjectsState projectsState = new ProjectsState();

    for ( ProjectState state : ProjectState.values() ) {
      if ( statusXml.contains( state.getHudsonColor() + "</color>" ) ) {
        projectsState.enable( state );
      }
    }
    return projectsState;
  }
}
